/*
	basic_type_decorator.hpp

	decorator for basic types
*/

#ifndef BOOST_DLL_BASIC_TYPE_DECORATOR_HPP
#define BOOST_DLL_BASIC_TYPE_DECORATOR_HPP

//#include "type_decorator.hpp"
#include "decorating_context.hpp"
#include <string>
#include <boost/static_assert.hpp>

namespace boost
{
	namespace detail
	{
		namespace dll
		{
			template<typename T>
			class type_decorator;

			template<typename T>
			class basic_type_decorator
			{
			public:
				basic_type_decorator() {}

				//
				// never used since specialisations covers all our needs
				std::string decorate(decorating_context &context) const { BOOST_STATIC_ASSERT(0); }
			};

			template<>
			class basic_type_decorator<void>
			{
			public:
				std::string decorate(decorating_context &context) const { return "X"; }
			};

			template<>
			class basic_type_decorator<char>
			{
			public:
				std::string decorate(decorating_context &context) const { return "D"; }
			};

			template<>
			class basic_type_decorator<signed char>
			{
			public:
				std::string decorate(decorating_context &context) const { return "D"; }
			};

			template<>
			class basic_type_decorator<unsigned char>
			{
			public:
				std::string decorate(decorating_context &context) const { return "E"; }
			};

			template<>
			class basic_type_decorator<signed short>
			{
			public:
				std::string decorate(decorating_context &context) const { return "F"; }
			};

			template<>
			class basic_type_decorator<unsigned short>
			{
			public:
				std::string decorate(decorating_context &context) const { return "G"; }
			};

			template<>
			class basic_type_decorator<signed int>
			{
			public:
				std::string decorate(decorating_context &context) const { return "H"; }
			};

			template<>
			class basic_type_decorator<unsigned int>
			{
			public:
				std::string decorate(decorating_context &context) const { return "I"; }
			};

			template<>
			class basic_type_decorator<signed long>
			{
			public:
				std::string decorate(decorating_context &context) const { return "J"; }
			};

			template<>
			class basic_type_decorator<unsigned long>
			{
			public:
				std::string decorate(decorating_context &context) const { return "K"; }
			};

			template<>
			class basic_type_decorator<float>
			{
			public:
				std::string decorate(decorating_context &context) const { return "M"; }
			};

			template<>
			class basic_type_decorator<double>
			{
			public:
				std::string decorate(decorating_context &context) const { return "N"; }
			};

			//
			// and so on
			//

			//
			// partial specialication for pointers
			template<typename T>
			class basic_type_decorator<T *>
			{
			public:
				std::string decorate(decorating_context &context) const { return std::string("PA") + base_decorator_.decorate(context); }

			private:
				type_decorator<T> base_decorator_;
			};

			//
			// partial specialication for const pointers
			template<typename T>
			class basic_type_decorator<const T *>
			{
			public:
				std::string decorate(decorating_context &context) const { return std::string("PB") + base_decorator_.decorate(context); }

			private:
				type_decorator<T> base_decorator_;
			};

			//
			// partial specialication for references
			template<typename T>
			class basic_type_decorator<T &>
			{
			public:
				std::string decorate(decorating_context &context) const { return std::string("AA") + base_decorator_.decorate(context); }

			private:
				type_decorator<T> base_decorator_;
			};

			//
			// partial specialication for const references
			template<typename T>
			class basic_type_decorator<const T &>
			{
			public:
				std::string decorate(decorating_context &context) const { return std::string("AB") + base_decorator_.decorate(context); }

			private:
				type_decorator<T> base_decorator_;
			};

		}; // end namespace dll
	}; // end namespace detail
}; // end namespace boost

#endif // BOOST_DLL_BASIC_TYPE_DECORATOR_HPP