#include "class_type_info_decorator.hpp"
#include "type_info_decorator.hpp"

namespace boost
{
namespace detail
{
namespace dll
{
class_type_info_decorator::class_type_info_decorator()
{
}

class_type_info_decorator::~class_type_info_decorator()
{
}

std::string class_type_info_decorator::decorate(decorating_context &context,
												const extended_type_info &info) const
{
	size_t pos = context.position(info);	
	std::string ret;
	if(pos == decorating_context::npos)
	{
		ret = decorate_class_type(context, info);

		if(!info.template_arguments().size())
		{
			ret += info.type_name();
			ret += decorate_namespaces(context, info);
			ret += "@@";
		}
		else
		{
			ret += "?$";
			ret += info.type_name();
			ret += decorate_template_arguments(context, info);
			ret += decorate_namespaces(context, info);
			ret += "@@";
		}

		context.push_back(info, ret);
	}
	else
	{
		ret = decorate_class_type(context, info);
		char buf[32];
		_snprintf(buf, sizeof(buf), "%u@", pos + 1); // need to add one since this position is one based
		ret += buf;
	}
	return ret;
}

std::string class_type_info_decorator::decorate_class_type(decorating_context &context,
														   const extended_type_info &info) const
{
	switch(info.type())
	{
	case extended_type_info::class_type:
		return "V";

	case extended_type_info::struct_type:
		return "U";

	default:
		BOOST_ASSERT(0);
		return "";
	}
}

std::string class_type_info_decorator::decorate_namespaces(decorating_context &context,
														   const extended_type_info &info) const
{
	std::string ret;
	for(std::vector<std::string>::const_iterator i = info.namespaces().begin(); i != info.namespaces().end(); ++i)
		ret += "@" + *i;

	return ret;
}

std::string class_type_info_decorator::decorate_template_arguments(decorating_context &context,
																   const extended_type_info &info) const
{
	std::string ret;

	for(std::vector< boost::shared_ptr<extended_type_info> >::const_iterator i = info.template_arguments().begin(); i != info.template_arguments().end(); ++i)
	{
		if((*i)->type() & extended_type_info::builtin_type_mask)
			ret += "@";
		ret += type_info_decorator().decorate(context, *(*i).get());
	}

	return ret;
}

}; // end namespace dll
}; // end namespace detail
}; // end namespace boost