/*
	dll.hpp
*/
#ifndef BOOST_DLL_HPP
#define BOOST_DLL_HPP

#include "dll_macros.hpp"
#include "type_decorator.hpp"
#include "decorating_context.hpp"
#include "function_descriptor.hpp"
#include <exception>
#include <boost/utility.hpp>

#include "win32_dll_handle.hpp"

namespace boost
{
	//
	// thrown by dll::call when no dll is loaded
	class invalid_dll : public std::runtime_error
	{
	public:
		invalid_dll() : std::runtime_error("no dll loaded") {}
	};

	//
	// thrown by dll::call when no such function can be called
	class unknown_function : public std::runtime_error
	{
	public:
		unknown_function() : std::runtime_error("no such function in dll") {}
	};

	class dll : public boost::noncopyable
	{
	public:
		dll() {}
		dll(const boost::filesystem::path &file)
		{
			if(!open(file))
				throw invalid_dll();
		}
		~dll() { handle_.close(); }

		bool open(const boost::filesystem::path &file)
		{
			// 
			// FIXME: exception safety?
			detail::dll::dll_handle handle(file);
			if(!handle)
				return false;

			if(handle_)
				handle_.close();

			handle_ = handle;
			return true;
		}

		//template<typename Signature>
		//	bool call(const std::string &functionName);

		BOOST_DLL_DECLARE_CALLS;

	private:
		detail::dll::dll_handle handle_;
	}; 
}; // end namespace dll

#endif // BOOST_DLL_HPP