/*
	win32_dll_handle.h

	DLL handle for WIN32 systems
*/
#ifndef BOOST_DLL_WIN32_DLL_HANDLE_HPP
#define BOOST_DLL_WIN32_DLL_HANDLE_HPP

#include <windows.h>
#include <string>
#include <boost/filesystem/path.hpp>

namespace boost
{
	namespace detail
	{
		namespace dll
		{
			//
			// note:
			//	handles are NOT automatically closed
			class dll_handle
			{
			public:
				typedef HMODULE handle_type;

				dll_handle() { handle_ = NULL; }
				dll_handle(HMODULE lib) : handle_(lib) {}
				dll_handle(const boost::filesystem::path &file) { load(file); }
				dll_handle(const dll_handle &h) : handle_(h.handle_) {}
				~dll_handle() {	}

				bool load(const boost::filesystem::path &file)
				{
					HMODULE handle = LoadLibrary(file.native_file_string().c_str());
					if(handle == NULL)
						return false;
					close();
					handle_ = handle;
					return true;
				}

				bool valid() const { return handle_ != NULL; }

				HMODULE handle() const { return handle_; }

				void *lookup(const std::string &symbol) const
				{
					if(handle_ == NULL)
						return 0;

					return GetProcAddress(handle_, symbol.c_str()); 
				}

				void close()
				{
					if(handle_ == NULL)
						return; // already closed
					
					FreeLibrary(handle_); // FIXME: check for error
					handle_ = NULL;
				}

				operator bool () { return valid(); }
				
				void *operator[] (const std::string &symbol) const
				{
					return lookup(symbol);
				}

				dll_handle &operator=(const dll_handle &h) 
				{
					close();
					handle_ = h.handle_;
					return *this;
				}

			private:
				HMODULE handle_;
			};

		}; // end namespace dll
	}; // end namespace detail
}; // end namespace boost

#endif // BOOST_DLL_WIN32_DLL_HANDLE_HPP